@@ -43,7 +43,7 @@ enum {
 	LAST_SIGNAL
 };
 
-static gint etcta_signals [LAST_SIGNAL] = { 0, };
+static guint etcta_signals [LAST_SIGNAL] = { 0 };
 
 #define PARENT_OBJECT_TYPE gnome_canvas_group_get_type ()
 
@@ -57,7 +57,7 @@ enum {
 	ARG_MODEL,
 	ARG_MESSAGE,
 	ARG_WIDTH,
-	ARG_HEIGHT,
+	ARG_HEIGHT
 };
 
 static void
@@ -301,6 +301,7 @@ finish_editing (ETableClickToAdd *etcta)
 	if (etcta->row) {
 		ETableModel *one;
 
+		e_table_item_leave_edit (E_TABLE_ITEM (etcta->row));
 		e_table_one_commit(E_TABLE_ONE(etcta->one));
 		etcta_drop_one (etcta);
 		gtk_object_destroy(GTK_OBJECT(etcta->row));
@@ -317,7 +318,7 @@ finish_editing (ETableClickToAdd *etcta)
 						   "ETableHeader", etcta->eth,
 						   "ETableModel", etcta->one,
 						   "minimum_width", etcta->width,
-						   "horizontal_draw_grid", FALSE,
+						   "horizontal_draw_grid", TRUE,
 						   "vertical_draw_grid", TRUE,
 						   "selection_model", etcta->selection,
 						   "cursor_mode", E_CURSOR_SPREADSHEET,
@@ -362,7 +363,7 @@ etcta_event (GnomeCanvasItem *item, GdkEvent *e)
 							   "ETableHeader", etcta->eth,
 							   "ETableModel", etcta->one,
 							   "minimum_width", etcta->width,
-							   "horizontal_draw_grid", FALSE,
+							   "horizontal_draw_grid", TRUE,
 							   "vertical_draw_grid", TRUE,
 							   "selection_model", etcta->selection,
 							   "cursor_mode", E_CURSOR_SPREADSHEET,
@@ -388,6 +389,7 @@ etcta_event (GnomeCanvasItem *item, GdkEvent *e)
 			return FALSE;
 			break;
 		}
+		break;
 			
 	default:
 		return FALSE;
