@@ -22,9 +22,14 @@
  * 02111-1307, USA.
  */
 
+
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <string.h>
+
 #include <gtk/gtksignal.h>
 #include <gnome-xml/tree.h>
 #include <gnome-xml/parser.h>
@@ -58,6 +63,7 @@ struct ETreeTableAdapterPriv {
 	int          tree_model_node_col_changed_id;
 	int          tree_model_node_inserted_id;
 	int          tree_model_node_removed_id;
+	int          tree_model_node_request_collapse_id;
 };
 
 typedef struct ETreeTableAdapterNode {
@@ -122,6 +128,26 @@ find_or_create_node(ETreeTableAdapter *etta, ETreePath path)
 	return node;
 }
 
+static gboolean
+e_tree_table_adapter_node_would_be_expanded (ETreeTableAdapter *etta, ETreePath path)
+{
+	ETreeTableAdapterNode *node;
+
+	if (e_tree_model_node_is_root(etta->priv->source, path) &&
+	    !etta->priv->root_visible)
+		return TRUE;
+
+	node = find_node (etta, path);
+	if (node)
+		return node->expanded;
+	else {
+		if (e_tree_model_node_is_root(etta->priv->source, path))
+			return TRUE;
+		else
+			return e_tree_model_get_expanded_default(etta->priv->source);
+	}
+}
+
 static void
 add_expanded_node(ETreeTableAdapter *etta, char *save_id, gboolean expanded)
 {
@@ -187,8 +213,7 @@ find_first_child_node_maybe_deleted(ETreeTableAdapter *adapter, int row)
 {
 	if (row != -1) {
 		ETreePath path = adapter->priv->map_table[row];
-		ETreeTableAdapterNode *current = find_node (adapter, path);
-		if (current && current->expanded) {
+		if (e_tree_table_adapter_node_would_be_expanded (adapter, path)) {
 			row ++;
 			if (row >= adapter->priv->n_map)
 				return -1;
@@ -208,7 +233,7 @@ find_next_node(ETreeTableAdapter *adapter, int row)
 		ETreeTableAdapterNode *current = find_node (adapter, path);
 		if (next_sibling) {
 			row += (current ? current->num_visible_children : 0) + 1;
-			if (row >= adapter->priv->n_map)
+			if (row > adapter->priv->n_map)
 				return -1;
 			return row;
 		} else
@@ -223,10 +248,9 @@ find_first_child_node(ETreeTableAdapter *adapter, int row)
 	if (row != -1) {
 		ETreePath path = adapter->priv->map_table[row];
 		ETreePath first_child = e_tree_model_node_get_first_child(adapter->priv->source, path);
-		ETreeTableAdapterNode *current = find_node (adapter, path);
-		if (first_child && current && current->expanded) {
+		if (first_child && e_tree_table_adapter_node_is_expanded (adapter, path)) {
 			row ++;
-			if (row >= adapter->priv->n_map)
+			if (row > adapter->priv->n_map)
 				return -1;
 			return row;
 		} else
@@ -289,12 +313,9 @@ find_row_num(ETreeTableAdapter *etta, ETreePath path)
 	sequence[0] = path;
 
 	for (i = 0; i < depth; i++) {
-		ETreeTableAdapterNode *node;
-		
 		sequence[i + 1] = e_tree_model_node_get_parent(etta->priv->source, sequence[i]);
 
-		node = find_node(etta, sequence[i + 1]);
-		if (! ((node && node->expanded) || e_tree_model_get_expanded_default(etta->priv->source))) {
+		if (!e_tree_table_adapter_node_is_expanded (etta, sequence[i + 1])) {
 			g_free(sequence);
 			return -1;
 		}
@@ -303,9 +324,11 @@ find_row_num(ETreeTableAdapter *etta, ETreePath path)
 	row = 0;
 
 	for (i = depth; i >= 0; i --) {
-		while (row != -1 && sequence[i] != etta->priv->map_table[row]) {
+		while (row != -1 && row < etta->priv->n_map && sequence[i] != etta->priv->map_table[row]) {
 			row = find_next_node(etta, row);
 		}
+		if (row >= etta->priv->n_map)
+			break;
 		if (row == -1)
 			break;
 		if (i == 0)
@@ -314,6 +337,9 @@ find_row_num(ETreeTableAdapter *etta, ETreePath path)
 	}
 	g_free (sequence);
 
+	if (row >= etta->priv->n_map)
+		row = -1;
+
 	d(g_print("Didn't find last access %d. Setting to %d. (find_row_num)\n", etta->priv->last_access, row));
 	etta->priv->last_access = row;
 	return row;
@@ -324,12 +350,7 @@ array_size_from_path(ETreeTableAdapter *etta, ETreePath path)
 {
 	int size = 1;
 
-	ETreeTableAdapterNode *node = NULL;
-
-	if (e_tree_model_node_is_expandable(etta->priv->source, path))
-		node = find_or_create_node(etta, path);
-
-	if (node && node->expanded) {
+	if (e_tree_table_adapter_node_is_expanded (etta, path)) {
 		ETreePath children;
 
 		for (children = e_tree_model_node_get_first_child(etta->priv->source, path); 
@@ -352,14 +373,14 @@ fill_array_from_path(ETreeTableAdapter *etta, ETreePath *array, ETreePath path)
 
 	index ++;
 
-	if (e_tree_model_node_is_expandable(etta->priv->source, path))
-		node = find_or_create_node(etta, path);
-	else
-		node = find_node(etta, path);
+	node = find_node(etta, path);
 
-	if (node && node->expanded) {
+	if (e_tree_table_adapter_node_is_expanded (etta, path)) {
 		ETreePath children;
 
+		if (!node)
+			node = find_or_create_node(etta, path);
+
 		for (children = e_tree_model_node_get_first_child(etta->priv->source, path); 
 		     children; 
 		     children = e_tree_model_node_get_next(etta->priv->source, children)) {
@@ -404,6 +425,8 @@ etta_destroy (GtkObject *object)
 				       etta->priv->tree_model_node_inserted_id);
 		gtk_signal_disconnect (GTK_OBJECT (etta->priv->source),
 				       etta->priv->tree_model_node_removed_id);
+		gtk_signal_disconnect (GTK_OBJECT (etta->priv->source),
+				       etta->priv->tree_model_node_request_collapse_id);
 
 		gtk_object_unref (GTK_OBJECT (etta->priv->source));
 		etta->priv->source = NULL;
@@ -415,6 +438,7 @@ etta_destroy (GtkObject *object)
 		etta->priv->tree_model_node_col_changed_id = 0;
 		etta->priv->tree_model_node_inserted_id = 0;
 		etta->priv->tree_model_node_removed_id = 0;
+		etta->priv->tree_model_node_request_collapse_id = 0;
 	}
 
 	g_free (etta->priv->map_table);
@@ -608,29 +632,30 @@ etta_class_init (ETreeTableAdapterClass *klass)
 static void
 etta_init (ETreeTableAdapter *etta)
 {
-	etta->priv                                  = g_new(ETreeTableAdapterPriv, 1);
+	etta->priv                                      = g_new(ETreeTableAdapterPriv, 1);
 
-	etta->priv->source                          = NULL;
+	etta->priv->source                              = NULL;
 
-	etta->priv->n_map                           = 0;
-	etta->priv->n_vals_allocated                = 0;
-	etta->priv->map_table                       = NULL;
-	etta->priv->attributes                      = NULL;
+	etta->priv->n_map                               = 0;
+	etta->priv->n_vals_allocated                    = 0;
+	etta->priv->map_table                           = NULL;
+	etta->priv->attributes                          = NULL;
 
-	etta->priv->root_visible                    = TRUE;
+	etta->priv->root_visible                        = TRUE;
 
-	etta->priv->last_access                     = 0;
+	etta->priv->last_access                         = 0;
 
-	etta->priv->tree_model_pre_change_id        = 0;
-	etta->priv->tree_model_no_change_id        = 0;
-	etta->priv->tree_model_node_changed_id      = 0;
-	etta->priv->tree_model_node_data_changed_id = 0;
-	etta->priv->tree_model_node_col_changed_id  = 0;
-	etta->priv->tree_model_node_inserted_id     = 0;
-	etta->priv->tree_model_node_removed_id      = 0;
+	etta->priv->tree_model_pre_change_id            = 0;
+	etta->priv->tree_model_no_change_id             = 0;
+	etta->priv->tree_model_node_changed_id          = 0;
+	etta->priv->tree_model_node_data_changed_id     = 0;
+	etta->priv->tree_model_node_col_changed_id      = 0;
+	etta->priv->tree_model_node_inserted_id         = 0;
+	etta->priv->tree_model_node_removed_id          = 0;
+	etta->priv->tree_model_node_request_collapse_id = 0;
 }
 
-E_MAKE_TYPE(e_tree_table_adapter, "ETreeTableAdapter", ETreeTableAdapter, etta_class_init, etta_init, PARENT_TYPE);
+E_MAKE_TYPE(e_tree_table_adapter, "ETreeTableAdapter", ETreeTableAdapter, etta_class_init, etta_init, PARENT_TYPE)
 
 static void
 etta_proxy_pre_change (ETreeModel *etm, ETreeTableAdapter *etta)
@@ -660,8 +685,10 @@ etta_proxy_node_changed (ETreeModel *etm, ETreePath path, ETreeTableAdapter *ett
 		int old_size;
 		ETreeTableAdapterNode *node;
 
-		if (row == -1)
+		if (row == -1) {
+			e_table_model_no_change(E_TABLE_MODEL(etta));
 			return;
+		}
 
 		size = array_size_from_path(etta, path);
 
@@ -735,6 +762,7 @@ etta_proxy_node_inserted (ETreeModel *etm, ETreePath parent, ETreePath child, ET
 		if (parent_node->expandable != e_tree_model_node_is_expandable(etta->priv->source, parent)) {
 			e_table_model_pre_change(E_TABLE_MODEL(etta));
 			parent_node->expandable = e_tree_model_node_is_expandable(etta->priv->source, parent);
+			parent_node->expandable_set = 1;
 			if (etta->priv->root_visible)
 				e_table_model_row_changed(E_TABLE_MODEL(etta), parent_row);
 			else if (parent_row != 0)
@@ -742,7 +770,8 @@ etta_proxy_node_inserted (ETreeModel *etm, ETreePath parent, ETreePath child, ET
 			else
 				e_table_model_no_change(E_TABLE_MODEL(etta));
 		}
-		if (!parent_node->expanded) {
+
+		if (!e_tree_table_adapter_node_is_expanded (etta, parent)) {
 			e_table_model_no_change(E_TABLE_MODEL(etta));
 			return;
 		}
@@ -784,7 +813,7 @@ etta_proxy_node_inserted (ETreeModel *etm, ETreePath parent, ETreePath child, ET
 }
 
 static void
-etta_proxy_node_removed (ETableModel *etm, ETreePath parent, ETreePath child, int old_position, ETreeTableAdapter *etta)
+etta_proxy_node_removed (ETreeModel *etm, ETreePath parent, ETreePath child, int old_position, ETreeTableAdapter *etta)
 {
 	int parent_row = find_row_num(etta, parent);
 	int row = find_child_row_num_maybe_deleted(etta, parent_row, child);
@@ -825,6 +854,12 @@ etta_proxy_node_removed (ETableModel *etm, ETreePath parent, ETreePath child, in
 		e_table_model_no_change(E_TABLE_MODEL(etta));
 }
 
+static void
+etta_proxy_node_request_collapse (ETreeModel *etm, ETreePath node, ETreeTableAdapter *etta)
+{
+	e_tree_table_adapter_node_set_expanded(etta, node, FALSE);
+}
+
 ETableModel *
 e_tree_table_adapter_construct (ETreeTableAdapter *etta, ETreeModel *source)
 {
@@ -847,20 +882,22 @@ e_tree_table_adapter_construct (ETreeTableAdapter *etta, ETreeModel *source)
 		fill_array_from_path(etta, etta->priv->map_table, root);
 	}
 
-	etta->priv->tree_model_pre_change_id = gtk_signal_connect (GTK_OBJECT (source), "pre_change",
-								   GTK_SIGNAL_FUNC (etta_proxy_pre_change), etta);
-	etta->priv->tree_model_no_change_id = gtk_signal_connect (GTK_OBJECT (source), "no_change",
-								   GTK_SIGNAL_FUNC (etta_proxy_no_change), etta);
-	etta->priv->tree_model_node_changed_id = gtk_signal_connect (GTK_OBJECT (source), "node_changed",
-								     GTK_SIGNAL_FUNC (etta_proxy_node_changed), etta);
-	etta->priv->tree_model_node_data_changed_id = gtk_signal_connect (GTK_OBJECT (source), "node_data_changed",
-									  GTK_SIGNAL_FUNC (etta_proxy_node_data_changed), etta);
-	etta->priv->tree_model_node_col_changed_id = gtk_signal_connect (GTK_OBJECT (source), "node_col_changed",
-									 GTK_SIGNAL_FUNC (etta_proxy_node_col_changed), etta);
-	etta->priv->tree_model_node_inserted_id = gtk_signal_connect (GTK_OBJECT (source), "node_inserted",
-								      GTK_SIGNAL_FUNC (etta_proxy_node_inserted), etta);
-	etta->priv->tree_model_node_removed_id = gtk_signal_connect (GTK_OBJECT (source), "node_removed",
-								     GTK_SIGNAL_FUNC (etta_proxy_node_removed), etta);
+	etta->priv->tree_model_pre_change_id            = gtk_signal_connect (GTK_OBJECT (source), "pre_change",
+									      GTK_SIGNAL_FUNC (etta_proxy_pre_change), etta);
+	etta->priv->tree_model_no_change_id             = gtk_signal_connect (GTK_OBJECT (source), "no_change",
+									      GTK_SIGNAL_FUNC (etta_proxy_no_change), etta);
+	etta->priv->tree_model_node_changed_id          = gtk_signal_connect (GTK_OBJECT (source), "node_changed",
+									      GTK_SIGNAL_FUNC (etta_proxy_node_changed), etta);
+	etta->priv->tree_model_node_data_changed_id     = gtk_signal_connect (GTK_OBJECT (source), "node_data_changed",
+									      GTK_SIGNAL_FUNC (etta_proxy_node_data_changed), etta);
+	etta->priv->tree_model_node_col_changed_id      = gtk_signal_connect (GTK_OBJECT (source), "node_col_changed",
+									      GTK_SIGNAL_FUNC (etta_proxy_node_col_changed), etta);
+	etta->priv->tree_model_node_inserted_id         = gtk_signal_connect (GTK_OBJECT (source), "node_inserted",
+									      GTK_SIGNAL_FUNC (etta_proxy_node_inserted), etta);
+	etta->priv->tree_model_node_removed_id          = gtk_signal_connect (GTK_OBJECT (source), "node_removed",
+									      GTK_SIGNAL_FUNC (etta_proxy_node_removed), etta);
+	etta->priv->tree_model_node_request_collapse_id = gtk_signal_connect (GTK_OBJECT (source), "node_request_collapse",
+									      GTK_SIGNAL_FUNC (etta_proxy_node_request_collapse), etta);
 
 	return E_TABLE_MODEL (etta);
 }
@@ -899,32 +936,33 @@ save_expanded_state_func (gpointer keyp, gpointer value, gpointer data)
 void
 e_tree_table_adapter_save_expanded_state (ETreeTableAdapter *etta, const char *filename)
 {
-	xmlDoc *doc;
-	xmlNode *root;
 	ETreeTableAdapterPriv *priv;
 	TreeAndRoot tar;
-
+	xmlDocPtr doc;
+	xmlNode *root;
+	
 	g_return_if_fail(etta != NULL);
 
 	priv = etta->priv; 
 
-	doc = xmlNewDoc ((xmlChar*) "1.0");
+	doc = xmlNewDoc ("1.0");
 	root = xmlNewDocNode (doc, NULL,
 			      (xmlChar *) "expanded_state",
 			      NULL);
 	xmlDocSetRootElement (doc, root);
 
-	e_xml_set_integer_prop_by_name(root, "vers", 1);
+	e_xml_set_integer_prop_by_name (root, "vers", 2);
+	e_xml_set_bool_prop_by_name (root, "default", e_tree_model_get_expanded_default (priv->source));
 
 	tar.root = root;
 	tar.tree = etta->priv->source;
-
+	
 	g_hash_table_foreach (priv->attributes,
 			      save_expanded_state_func,
 			      &tar);
-
-	xmlSaveFile (filename, doc);
-
+	
+	e_xml_save_file (filename, doc);
+	
 	xmlFreeDoc (doc);
 }
 
@@ -936,6 +974,9 @@ e_tree_table_adapter_load_expanded_state (ETreeTableAdapter *etta, const char *f
 	xmlNode *root;
 	xmlNode *child;
 	int vers;
+	gboolean model_default, saved_default;
+	int size;
+	ETreePath path;
 
 	g_return_if_fail(etta != NULL);
 
@@ -951,11 +992,19 @@ e_tree_table_adapter_load_expanded_state (ETreeTableAdapter *etta, const char *f
 		return;
 	}
 
-	vers = e_xml_get_integer_prop_by_name_with_default(root, "vers", 0);
-	if (vers != 1) {
+	vers = e_xml_get_integer_prop_by_name_with_default (root, "vers", 0);
+	if (vers > 2) {
 		xmlFreeDoc (doc);
 		return;
 	}
+	model_default = e_tree_model_get_expanded_default (priv->source);
+	saved_default = e_xml_get_bool_prop_by_name_with_default (root, "default", !model_default);
+	if (saved_default != model_default) {
+		xmlFreeDoc (doc);
+		return;
+	}
+
+	e_table_model_pre_change(E_TABLE_MODEL(etta));
 
 	for (child = root->xmlChildrenNode; child; child = child->next) {
 		char *id;
@@ -969,13 +1018,22 @@ e_tree_table_adapter_load_expanded_state (ETreeTableAdapter *etta, const char *f
 
 		if (!strcmp(id, "")) {
 			g_free(id);
-			return;
+			continue;
 		}
 
-		add_expanded_node(etta, id, !e_tree_model_get_expanded_default(etta->priv->source));
+		add_expanded_node(etta, id, !model_default);
 	}
+	
 
 	xmlFreeDoc (doc);
+
+	path = e_tree_model_get_root (etta->priv->source);
+	size = array_size_from_path(etta, path);
+	etta_expand_to(etta, size);
+	etta->priv->n_map = size;
+	fill_array_from_path(etta, etta->priv->map_table, path);
+
+	e_table_model_changed (E_TABLE_MODEL (etta));
 }
 
 void         e_tree_table_adapter_root_node_set_visible (ETreeTableAdapter *etta, gboolean visible)
@@ -999,57 +1057,58 @@ void         e_tree_table_adapter_node_set_expanded (ETreeTableAdapter *etta, ET
 	ETreeTableAdapterNode *node;
 	int row;
 
-	if (e_tree_model_node_is_root (etta->priv->source, path) && !etta->priv->root_visible)
+	if (!expanded && e_tree_model_node_is_root (etta->priv->source, path) && !etta->priv->root_visible)
 		return;
 
 	node = find_or_create_node(etta, path);
 
-	if (expanded != node->expanded) {
-		node->expanded = expanded;
+	if (expanded == node->expanded)
+		return;
+
+	node->expanded = expanded;
 		
-		row = find_row_num(etta, path);
-		if (row != -1) {
+	row = find_row_num(etta, path);
+	if (row != -1) {
+		e_table_model_pre_change (E_TABLE_MODEL(etta));
+
+		if (etta->priv->root_visible) {
 			e_table_model_pre_change (E_TABLE_MODEL(etta));
+			e_table_model_row_changed(E_TABLE_MODEL(etta), row);
+		} else if (row != 0) {
+			e_table_model_pre_change (E_TABLE_MODEL(etta));
+			e_table_model_row_changed(E_TABLE_MODEL(etta), row - 1);
+		}
 
-			if (etta->priv->root_visible) {
-				e_table_model_pre_change (E_TABLE_MODEL(etta));
-				e_table_model_row_changed(E_TABLE_MODEL(etta), row);
-			} else if (row != 0) {
-				e_table_model_pre_change (E_TABLE_MODEL(etta));
-				e_table_model_row_changed(E_TABLE_MODEL(etta), row - 1);
-			}
-
-			if (expanded) {
-				int num_children = array_size_from_path(etta, path) - 1;
-				etta_expand_to(etta, etta->priv->n_map + num_children);
-				memmove(etta->priv->map_table + row + 1 + num_children,
-					etta->priv->map_table + row + 1,
-					(etta->priv->n_map - row - 1) * sizeof (ETreePath));
-				fill_array_from_path(etta, etta->priv->map_table + row, path);
-				etta_update_parent_child_counts(etta, path, num_children);
-				if (num_children != 0) {
-					if (etta->priv->root_visible)
-						e_table_model_rows_inserted(E_TABLE_MODEL(etta), row + 1, num_children);
-					else
-						e_table_model_rows_inserted(E_TABLE_MODEL(etta), row, num_children);
-				} else
-					e_table_model_no_change(E_TABLE_MODEL(etta));
-			} else {
-				int num_children = node->num_visible_children;
-				g_assert (etta->priv->n_map >= row + 1 + num_children);
-				memmove(etta->priv->map_table + row + 1,
-					etta->priv->map_table + row + 1 + num_children,
-					(etta->priv->n_map - row - 1 - num_children) * sizeof (ETreePath));
-				node->num_visible_children = 0;
-				etta_update_parent_child_counts(etta, path, - num_children);
-				if (num_children != 0) {
-					if (etta->priv->root_visible)
-						e_table_model_rows_deleted(E_TABLE_MODEL(etta), row + 1, num_children);
-					else
-						e_table_model_rows_deleted(E_TABLE_MODEL(etta), row, num_children);
-				} else
-					e_table_model_no_change(E_TABLE_MODEL(etta));
-			}
+		if (expanded) {
+			int num_children = array_size_from_path(etta, path) - 1;
+			etta_expand_to(etta, etta->priv->n_map + num_children);
+			memmove(etta->priv->map_table + row + 1 + num_children,
+				etta->priv->map_table + row + 1,
+				(etta->priv->n_map - row - 1) * sizeof (ETreePath));
+			fill_array_from_path(etta, etta->priv->map_table + row, path);
+			etta_update_parent_child_counts(etta, path, num_children);
+			if (num_children != 0) {
+				if (etta->priv->root_visible)
+					e_table_model_rows_inserted(E_TABLE_MODEL(etta), row + 1, num_children);
+				else
+					e_table_model_rows_inserted(E_TABLE_MODEL(etta), row, num_children);
+			} else
+				e_table_model_no_change(E_TABLE_MODEL(etta));
+		} else {
+			int num_children = node->num_visible_children;
+			g_assert (etta->priv->n_map >= row + 1 + num_children);
+			memmove(etta->priv->map_table + row + 1,
+				etta->priv->map_table + row + 1 + num_children,
+				(etta->priv->n_map - row - 1 - num_children) * sizeof (ETreePath));
+			node->num_visible_children = 0;
+			etta_update_parent_child_counts(etta, path, - num_children);
+			if (num_children != 0) {
+				if (etta->priv->root_visible)
+					e_table_model_rows_deleted(E_TABLE_MODEL(etta), row + 1, num_children);
+				else
+					e_table_model_rows_deleted(E_TABLE_MODEL(etta), row, num_children);
+			} else
+				e_table_model_no_change(E_TABLE_MODEL(etta));
 		}
 	}
 }
@@ -1105,17 +1164,16 @@ void         e_tree_table_adapter_show_node (ETreeTableAdapter *etta, ETreePath
 
 	parent = e_tree_model_node_get_parent(etta->priv->source, path);
 
-	if (parent) {
+	while (parent) {
 		e_tree_table_adapter_node_set_expanded(etta, parent, TRUE);
-		e_tree_table_adapter_show_node(etta, parent);
+		parent = e_tree_model_node_get_parent(etta->priv->source, parent);
 	}
 }
 
 gboolean     e_tree_table_adapter_node_is_expanded (ETreeTableAdapter *etta, ETreePath path)
 {
-	if (e_tree_model_node_is_expandable(etta->priv->source, path)) {
-		ETreeTableAdapterNode *node = find_or_create_node(etta, path);
-		return node->expanded;
-	} else
+	if (!e_tree_model_node_is_expandable (etta->priv->source, path))
 		return FALSE;
+
+	return e_tree_table_adapter_node_would_be_expanded (etta, path);
 }
